@@ -1,8 +1,8 @@
 import numpy as np
 
-from .utils import check_boolean, funcs_no_separate_nan, get_func, aggregate_common_doc, isstr
-from .utils_numpy import (aliasing, minimum_dtype, input_validation,
-                          check_dtype, check_fill_value, minimum_dtype_scalar)
+from .utils import aggregate_common_doc, check_boolean, funcs_no_separate_nan, get_func, isstr
+from .utils_numpy import (aliasing, check_dtype, check_fill_value, input_validation, iscomplexobj,
+                          minimum_dtype, minimum_dtype_scalar)
 
 
 def _sum(group_idx, a, size, fill_value, dtype=None):
@@ -13,12 +13,12 @@ def _sum(group_idx, a, size, fill_value, dtype=None):
         if a != 1:
             ret *= a
     else:
-        if np.iscomplexobj(a):
+        if iscomplexobj(a):
             ret = np.empty(size, dtype=dtype)
             ret.real = np.bincount(group_idx, weights=a.real,
-                              minlength=size)
+                                   minlength=size)
             ret.imag = np.bincount(group_idx, weights=a.imag,
-                              minlength=size)
+                                   minlength=size)
         else:
             ret = np.bincount(group_idx, weights=a,
                               minlength=size).astype(dtype)
@@ -128,7 +128,7 @@ def _mean(group_idx, a, size, fill_value, dtype=np.dtype(np.float64)):
     if np.ndim(a) == 0:
         raise ValueError("cannot take mean with scalar a")
     counts = np.bincount(group_idx, minlength=size)
-    if np.iscomplexobj(a):
+    if iscomplexobj(a):
         dtype = a.dtype  # TODO: this is a bit clumsy
         sums = np.empty(size, dtype=dtype)
         sums.real = np.bincount(group_idx, weights=a.real,
@@ -152,10 +152,11 @@ def _var(group_idx, a, size, fill_value, dtype=np.dtype(np.float64),
         raise ValueError("cannot take variance with scalar a")
     counts = np.bincount(group_idx, minlength=size)
     sums = np.bincount(group_idx, weights=a, minlength=size)
-    with np.errstate(divide='ignore'):
+    with np.errstate(divide='ignore', invalid='ignore'):
         means = sums.astype(dtype) / counts
+        counts = np.where(counts > ddof, counts - ddof, 0)
         ret = np.bincount(group_idx, (a - means[group_idx]) ** 2,
-                          minlength=size) / (counts - ddof)
+                          minlength=size) / counts
     if sqrt:
         ret = np.sqrt(ret)  # this is now std not var
     if not np.isnan(fill_value):
@@ -253,17 +254,18 @@ _impl_dict.update(('nan' + k, v) for k, v in list(_impl_dict.items())
 def _aggregate_base(group_idx, a, func='sum', size=None, fill_value=0,
                     order='C', dtype=None, axis=None, _impl_dict=_impl_dict,
                     _nansqueeze=False, cache=None, **kwargs):
-    group_idx, a, flat_size, ndim_idx, size = input_validation(group_idx, a,
-                                             size=size, order=order, axis=axis)
+    group_idx, a, flat_size, ndim_idx, size, unravel_shape = input_validation(group_idx, a,
+                                                                              size=size, order=order, axis=axis, func=func)
+
     if group_idx.dtype == np.dtype("uint64"):
         # Force conversion to signed int, to avoid issues with bincount etc later
         group_idx = group_idx.astype(int)
-        
+
     func = get_func(func, aliasing, _impl_dict)
     if not isstr(func):
         # do simple grouping and execute function in loop
         ret = _impl_dict.get('generic', _generic_callable)(group_idx, a, flat_size, fill_value, func=func,
-                                dtype=dtype, **kwargs)
+                                                           dtype=dtype, **kwargs)
     else:
         # deal with nans and find the function
         if func.startswith('nan'):
@@ -282,6 +284,12 @@ def _aggregate_base(group_idx, a, func='sum', size=None, fill_value=0,
 
     # deal with ndimensional indexing
     if ndim_idx > 1:
+        if unravel_shape is not None:
+            # A negative fill_value cannot, and should not, be unraveled.
+            mask = ret == fill_value
+            ret[mask] = 0
+            ret = np.unravel_index(ret, unravel_shape)[axis]
+            ret[mask] = fill_value
         ret = ret.reshape(size, order=order)
     return ret
 
@@ -292,12 +300,12 @@ def aggregate(group_idx, a, func='sum', size=None, fill_value=0, order='C',
                            order=order, dtype=dtype, func=func, axis=axis,
                            _impl_dict=_impl_dict, _nansqueeze=True, **kwargs)
 
+
 aggregate.__doc__ = """
     This is the pure numpy implementation of aggregate.
     """ + aggregate_common_doc
 
 
-
 def _fill_untouched(idx, ret, fill_value):
     """any elements of ret not indexed by idx are set to fill_value."""
     untouched = np.ones_like(ret, dtype=bool)
